import logging

from wooly import Widget
from wooly.template import WidgetTemplate
from wooly.util import StringCatalog, Writer

from cumin.objectframe import ObjectFrame, ObjectView
from cumin.parameters import CollectorGridAttribute
from cumin.stat import StatFlashChart, StatSet
from cumin.grid.dashboard import PoolDashboard

from submission import SubmissionFrame, PoolSubmissionJoinSelector
from slot import SlotFrame
from scheduler import SchedulerFrame, SchedulerSelector
from negotiator import NegotiatorFrame, NegotiatorSelector,\
    NegotiatorEditDynamicQuota, NegotiatorAttribute
from limit import LimitFrame, LimitSelector
from cumin.grid.quota import QuotaSelector 
from collector import CollectorGeneralStatSet
from cumin.grid.slotvis import SlotOverview

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.pool")

class PoolView(ObjectView):
    def __init__(self, app, name, object):
        super(PoolView, self).__init__(app, name, object)

        self.error_tmpl = WidgetTemplate(self, "error_html")

    def do_render(self, session):
        id = self.frame.id.get(session)
        if not id:
            writer = Writer()
            self.error_tmpl.render(writer, session)
            return writer.to_string()

        return super(PoolView, self).do_render(session)

    def render_title(self, session):
        obj = self.object.get(session)
        if not obj:
            return "Missing Pool"

        return obj.get_title()

class PoolFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector

        super(PoolFrame, self).__init__(app, name, cls)

        self.icon_href = "resource?name=pool-36.png"

        self.view = PoolView(app, "view", self.object)
        self.replace_child(self.view)

        self.mode.default = self.view

        self.submission = SubmissionFrame(app, "submission")
        self.add_mode(self.submission)

        self.slot = SlotFrame(app, "slot")
        self.add_mode(self.slot)

        self.scheduler = SchedulerFrame(app, "scheduler", self.object)
        self.add_mode(self.scheduler)

        self.negotiator = NegotiatorFrame(app, "negotiator")
        self.add_mode(self.negotiator)

        self.limit = LimitFrame(app, "limit")
        self.add_mode(self.limit)

        dashboard = PoolDashboard(app, "dashboard", self.object)
        self.view.add_tab(dashboard)

        submissions = PoolSubmissionJoinSelector(app, "pool_submissions", self.object)
        self.view.add_tab(submissions)

        slots = SlotOverview(app, "slots", self.object)
        self.view.add_tab(slots)

        schedulers = SchedulerSelector(app, "schedulers", self.object)
        self.view.add_tab(schedulers)

        negotiators = NegotiatorSelector(app, "negotiators", self.object)
        self.view.add_tab(negotiators)

        self.negotiator_attribute = NegotiatorAttribute(app, "neg", self.object)
        self.add_attribute(self.negotiator_attribute)

        self.edit_dynamic_quota = NegotiatorEditDynamicQuota(app, self)
        self.quotas = QuotaSelector(app, "quotas", self.negotiator_attribute, self)
        self.view.add_tab(self.quotas)

        self.limits = LimitSelector(app, "limits", self.object)
        self.view.add_tab(self.limits)

        self.top_tab = True

    def render_title(self, session):
        return "Grid"

    def get_title(self, session):
        obj = self.object.get(session)
        if not obj:
            return None

        return super(PoolFrame, self).get_title(session)

    def do_process(self, session):
        id = self.id.get(session)
        if not id:
            # get all the collectors
            collectors = self.cls.get_selection(session.cursor)
            try:
                youngest = collectors[0]
            except IndexError:
                return

            for collector in collectors:
                if collector._qmf_update_time > youngest._qmf_update_time:
                    youngest = collector
            collector = youngest
            self.id.set(session, collector._id)

        super(PoolFrame, self).do_process(session)

class PoolOverview(Widget):
    def __init__(self, app, name, collector):
        super(PoolOverview, self).__init__(app, name)

        self.collector = collector

        self.grid = CollectorGridAttribute(app, "grid", self.collector)
        self.add_attribute(self.grid)

        stats = CollectorGeneralStatSet(app, "collector_stats", self.collector)
        self.add_child(stats)

        stats = GridStats(app, "grid_stats", self.grid)
        self.add_child(stats)

        chart = self.JobStackedChart(app, "jobs", self.collector)
        chart.duration.param.default = "3600"
        chart.stats = ("RunningJobs", "IdleJobs")
        chart.chart_type = "stacked"
        self.add_child(chart)

        chart = self.SlotFlashChart(app, "slots", self.collector)
        chart.duration.param.default = "3600"
        chart.stats = ("HostsClaimed", "HostsUnclaimed", "HostsOwner")
        chart.chart_type = "stacked"
        self.add_child(chart)

    def render_title(self, session):
        return "Overview"

    def render_pool_name(self, session):
        collector = self.collector.get(session)
        return collector.Name

    class JobStackedChart(StatFlashChart):
        def render_title(self, session):
            return "Job status"

    class SlotFlashChart(StatFlashChart):
        def render_title(self, session):
            return "Slot state"

class GridGeneralStatSet(StatSet):
    def __init__(self, app, name, object):
        super(GridGeneralStatSet, self).__init__(app, name, object)

        self.attrs = ("NumJobs", "SubmitsInProgress",
                      "SubmitsQueued", "SubmitsAllowed",
                      "SubmitsWanted", "RunningJobs", "IdleJobs")

    def do_render(self, session):
        grid = self.object.get(session)

        if grid:
            return super(GridGeneralStatSet, self).do_render(session)

class GridStats(Widget):
    def __init__(self, app, name, grid):
        super(GridStats, self).__init__(app, name)

        self.grid = grid

        stats = GridGeneralStatSet(app, "stats", self.grid)
        self.add_child(stats)

        chart = self.GridJobChart(app, "jobs_chart", self.grid)
        chart.stats = ("NumJobs", "RunningJobs", "IdleJobs")
        self.add_child(chart)

        chart = self.GridSubmitChart(app, "submit_chart", self.grid)
        chart.stats = ("SubmitsAllowed", "SubmitsQueued", "SubmitsInProgress")
        self.add_child(chart)

    def render_grid_name(self, session):
        grid = self.grid.get(session)

        if grid:
            return grid.Name

    def do_render(self, session):
        grid = self.grid.get(session)

        if grid:
            return super(GridStats, self).do_render(session)

    class GridJobChart(StatFlashChart):
        def render_title(self, session):
            return "Grid jobs"

    class GridSubmitChart(StatFlashChart):
        def render_title(self, session):
            return "Grid submits"
