import logging
import math

from wooly import Session, Widget
from wooly.util import StringCatalog
from wooly.widgets import RadioModeSet

from slot import SlotMap, SlotMapPage
from cumin.parameters import RosemaryObjectParameter
from cumin.widgets import StateSwitch
from cumin.stat import FlashFullPage
from cumin.objectselector import ObjectSelector, ObjectLinkColumn,\
    ObjectTableColumn, ObjectTable

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.slotvis")

class SlotOverview(RadioModeSet):
    def __init__(self, app, name, pool):
        super(SlotOverview, self).__init__(app, name)

        self.pool = pool

        self.add_tab(SlotSelector(app, "slots", pool))
        self.add_tab(Visualization(app, "vis", pool))

    def render_title(self, session):
        return "Slots"

class SlotSelector(ObjectSelector):
    def __init__(self, app, name, pool):
        cls = app.model.com_redhat_grid.Slot

        super(SlotSelector, self).__init__(app, name, cls)

        self.pool = pool

        self.add_filter(self.pool, cls.Pool, cls.Pool)

        frame = "main.grid.slot"
        col = ObjectLinkColumn(app, "name", cls.Name, cls._id, frame)
        self.add_column(col)
        self.add_search_filter(col)

        self.add_attribute_column(cls.Activity)
        self.add_attribute_column(cls.State)

        col = ObjectTableColumn(app, cls.LoadAvg.name, cls.LoadAvg)
        self.add_column(col)
        # sort by load average descending
        self.table.sort.default = col.name
        self.table.ascending.default = False

        self.enable_csv_export(pool)

    def create_table(self, app, name, cls):
        # avoid the checkboxes
        return ObjectTable(app, name, cls)

    def render_title(self, session):
        return "Slot table"

class Visualization(Widget):
    def __init__(self, app, name, collector):
        super(Visualization, self).__init__(app, name)

        self.collector = collector

        slot_map = PoolSlotMap(app, "slot_png", self.collector)
        self.add_child(slot_map)

        slot_vis = PoolSlotFlashVis(app, "slot_map", self.collector)
        self.add_child(slot_vis)

    def render_title(self, session):
        return "Slot visualization"

class PoolSlotMap(SlotMap):
    def __init__(self, app, name, collector):
        super(PoolSlotMap, self).__init__(app, name)

        self.collector = collector

    def do_process(self, session):
        super(PoolSlotMap, self).do_process(session)

        collector = self.collector.get(session)

        self.slots.add_where_expr(session, "\"Pool\" = '%s'", collector.Pool)

    def render_image_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    def get_scheduler_select(self, session):
        pass
        #pool = self.pool.get(session)

class PoolSlotMapPage(SlotMapPage):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector
        self.collector = RosemaryObjectParameter(app, "id", cls)

        super(PoolSlotMapPage, self).__init__(app, name, self.collector, "Pool")

        self.add_parameter(self.collector)

    def do_process(self, session):
        super(PoolSlotMapPage, self).do_process(session)

        collector = self.collector.get(session)

        # XXX The conditional is necessary because this page is
        # overloaded to generate little dots, in which case it has no
        # pool

        if collector:
            self.slots.add_where_expr(session, "\"Pool\" = '%s'", collector.Pool)

class PoolSlotFlashVis(PoolSlotMap):
    def __init__(self, app, name, object):
        super(PoolSlotFlashVis, self).__init__(app, name, object)

        self.group_by = self.GroupBySwitch(app, "group_by")
        self.add_child(self.group_by)

        self.fullpageable = True

    def render_slots_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)
        page.json.set(sess, "slots")
        page.group.set(sess, self.group_by.get(session))

        return sess.marshal()

    def render_image_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    def render_slot_chart_width(self, session):
        return 740

    def render_slot_chart_height(self, session):
        cursor = self.slots.execute(session)
        data = cursor.fetchall()

        slot_size = 51
        count = len(data)
        # individual slots are 51 px wide and tall
        width = self.render_slot_chart_width(session)
        cols = int(width / slot_size) * 4
        rows = int(math.ceil(count / cols))

        return min(max(rows * slot_size, width), 4095)

    def render_fullpageable(self, session):
        return self.fullpageable and "fullpageable" or ""

    def render_fullpage_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_fullpage
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    class GroupBySwitch(StateSwitch):
        def __init__(self, app, name):
            super(PoolSlotFlashVis.GroupBySwitch, self).__init__(app, name)

            self.add_state("Activity|State", "By activity/state")
            self.add_state("AccountingGroup", "By accounting group")
            
        def get_click(self, session, state):
            href = self.parent.render_slots_href(session)
            return "return slot_vis('%s', this, '%s', '%s')" % \
                (state, self.parent.path, href)

class PoolSlotFullPage(FlashFullPage):
    def __init__(self, app, name):
        super(PoolSlotFullPage, self).__init__(app, name)

        cls = app.model.com_redhat_grid.Collector
        self.collector = RosemaryObjectParameter(app, "id", cls)
        self.add_parameter(self.collector)

        self.flash_chart = PoolSlotFlashVis(app, "chart", self.collector)
        self.flash_chart.fullpageable = False
        self.replace_child(self.flash_chart)

    def render_content(self, session):
        return self.flash_chart.render(session)

